/*
 **************************************************************************************************
 *
 *     PruefzifferValidate.java
 *     (c) 2001 by imMEDIAtely Medienversand, Germany
 *
 *     First created: 23.03.2001
 *     Last change:   02.05.2001
 *
 *     This software is part of PruefzifferValidate Java Edition
 *
 *     This programm is intended to demonstrate the use of the package de.pruefziffer.validate
 *     with the classes AccountValidator, CreditcardValidator, ISBNValidator and EANValidator.
 *
 *     This sourcecode and the compiled software are protected by international copyright.
 *
 **************************************************************************************************/


import java.io.*;

import de.pruefziffer.validate.*;

class PruefzifferValidate {


/*
 **************************************************************************************************
 * 
 *     MAIN METHOD
 *
 **************************************************************************************************/

	public static void main (String[] args) {
  		
  		PruefzifferValidate Validate = new PruefzifferValidate ();
  		
  		Validate.doRequestedAction (args);
  	}
  	
  	
/*
 **************************************************************************************************
 * 
 *     PRIVATE METHODS
 *
 **************************************************************************************************/


	private void doRequestedAction (String[] args) {
  		switch (getOption (args, 1)) {
  			case 'b':
  				bankaccount (args);
  				break;
  			case 'c':
  				showHelp(4, "Creditcard check is not included in evaluation version");
  				break;
  			case 'i':
  				showHelp(4, "ISBN check / calculation is not included in evaluation version");
  				break;
  			case 'e':
  				showHelp(4, "EAN check / calculation is not included in evaluation version");
  				break;
  			case 'd':
				  listDataFileInformation ();
				  break;
			  case 'n':
				  showHelp(4, "IBAN check is not included in evaluation version");
				  break;
  			default:
  				showHelp (0);
  		}		
	}
	

/* Extracts from the argumentarray the n-th option
 * start counting with 1
 * an option must start with '-'
 * all n options must be in first n fields of the array args
 */
	private char getOption (String[] args, int nthOption) {
		char retVal = ' ';
		if (args.length >= nthOption)
			if (args[nthOption - 1].length () >= 2)
	  			if (args[nthOption - 1].charAt (0) == '-')  // if not it's not an option!
  					retVal = Character.toLowerCase (args[nthOption - 1].charAt (1));
		return retVal;
	}
	

/* Method to validate a bankaccountnumber
 */
  	private void bankaccount (String[] args) {
  		
		// Variables for options to set
  		int strictness = AccountValidator.VALIDATE_STRICTNESS_NORMAL;
		boolean batchValidation = false;
		int caching = AccountValidator.BANKDATA_CACHE_OFF;
		boolean dataFileInformation = false;
        boolean search = false;

		// it's also possible to check a bankaccountnumber with a known methodnumber
		// instead of the banknumber -> banknumberTest = false
  		boolean banknumberTest = true;
  		
  		
  		// evaluate all options
  		int argNr = 1;
  		char option = getOption (args, argNr + 1);
  		while (option != ' ') {
			switch (option) {
				case 'a':
					caching = AccountValidator.BANKDATA_CACHE_ALL;
					break;
				case 'b':
					batchValidation = true;
					break;
				case 'c':
					caching = AccountValidator.BANKDATA_CACHE_METHODS;
					break;
				case 'l':
					strictness = AccountValidator.VALIDATE_STRICTNESS_LOW;
					break;
				case 'n':
					strictness = AccountValidator.VALIDATE_STRICTNESS_NORMAL;
					break;
				case 's':
					strictness = AccountValidator.VALIDATE_STRICTNESS_STRICT;
					break;
				case 'r':
					strictness = AccountValidator.VALIDATE_STRICTNESS_RESTRICTIVE;
					break;
                case 'f':
                    search = true;
                    break;
			}
  			option = getOption (args, (++argNr) + 1);
  		}

  		// New instance of AccountValidator-Class with option for bankdata-caching
		AccountValidator validator = new AccountValidator(caching);

		validator.setStrictness (strictness);

        if (search) {
            if (validator.bankSearch (args[argNr]) == 0)
                System.out.println ("\nEs wurden keine passende Banken gefunden!");
            else {
                validator.firstFoundBank ();
                do {
                    System.out.println(validator.getBankNumber () + " " + validator.getBankName () + " " + validator.getBankCity ()
                        + (validator.hasEnhancedBankData() ? " (" + validator.getBIC() + ")" : ""));
                } while (validator.nextFoundBank ());
                System.out.println (validator.getNumberFoundBanks () + " Banken gefunden");
            }            
            return;
        }
  		
  		// beside the options we need an account- and a banknumber for validation
  		// or a input- and outputfile for batchvalidation
  		if (args.length < argNr + 2) {
  			if (!batchValidation)
	  			showHelp (4, "Please specify a bankaccount- and banknumber for validation");
	  		else
	  			showHelp (4, "To start a batch-validation please specify an input- and outputfile\n");

  			return;
		} 		
		
		if (batchValidation) {
			fileCheck (validator, args[argNr], args[argNr + 1]);
			return;
		}        
		
		// New variables for bank- and accountnumber
		String banknumber = args[argNr];
		String accountnumber = args[argNr + 1];

		// A german banknumber must have 8 digits, only 1 or 2 digits means to test with the
		// testmethodnumber instead of a banknumber
		if (banknumber.length () < 4)
			banknumberTest = false;
			
			validator.setCharacterEncoding("ISO-8859-1");
  		
  		// Validate the accountnumber either with the banknumber or the testmethodnumber
		boolean isOkay = false;
		if (banknumberTest)
			isOkay = validator.validate (banknumber, accountnumber);
		else
			isOkay = validator.validate (Integer.parseInt (banknumber), accountnumber);

  		// Print the result of the validation to screen
		System.out.println ("\nThe accountnumber is " + ((isOkay)?"":"NOT ") + "valid.");
		System.out.println ("  accountnumber: " + validator.getAccountNumber ());
		System.out.println ("  banknumber:    " + validator.getBankNumber ());
		if (isOkay) {
		  System.out.println ("  iban:          " + validator.getIBAN());
	  }
		
		// if result is okay print the name of the bank to screen, else an errormessage
  	if (validator.hasEnhancedBankData()) {
		  System.out.println ("  bic:           " + validator.getBIC());
		  System.out.println ("  bank: " + validator.getBankName () + ", " 
           + validator.getBankZipCode() + " " + validator.getBankCity ());
      if (validator.isMarkedForDeletion()) {
        System.out.println("  BANK IS MARKED FOR DELETION! Replacement banknumber: "
            + validator.getReplacementBanknumber());         
      }
    } else {
      System.out.println ("  bank: " + validator.getBankName () + ", " + validator.getBankCity ());      
    }
		if (!isOkay) {
			System.out.println ("  " + validator.getLastErrorText ());
		}
		if (validator.isSavingsAccount()) {
			System.out.println ("SAVINGS-ACCOUNT!");
			disclaimer (8);
		} else 
			disclaimer (7);
	}

	
	private void showHelp (int topic) { showHelp (topic, ""); }
	
	private void showHelp (int topic, String error) {
		
		int offset = 0;
		
		if (error.length () > 0) {
			System.out.println ("\n" + error);
			offset = 2;
		}
			
		switch (topic) {
			case 0:
				System.out.println ("\n  PruefzifferValidate is a tool to validate the following numbertypes:");
				System.out.println ("    German bankaccountnumbers");
				System.out.println ("    International bankaccountnumbers (IBAN)");
				System.out.println ("    Creditcardnumbers");
				System.out.println ("    ISBN (can also be calculated)");
				System.out.println ("    EAN13 (can also be calculated)");
				System.out.println ("\n  use PruefzifferValidate with one option:");
				System.out.println ("    -b  to validate a bankaccount");
				System.out.println ("    -n  to validate/calculate IBAN");
				System.out.println ("    -c  to validate a creditcardnumber");
				System.out.println ("    -i  to validate/calculate ISBN");
				System.out.println ("    -e  to validate/calculate EAN13");
				System.out.println ("    -d  to display information about used banknumberfile");
				System.out.println ("  A call of PruefzifferValidate with one of the options above");
				System.out.println ("  will show more information for the option");
				
				disclaimer (16 + offset);
				break;
			case 1:  // Help for ISBN calculation and validation
				System.out.println ("\nISBN validation/calculation");
				System.out.println ("\n  use -v for validation, -c for calculation or -e for transformation to EAN13");
				System.out.println ("\n  e.g. validate an ISBN:");
				System.out.println ("    java PruefzifferValidate -i -v 38234585291");
				System.out.println ("  calculate the checkdigit for an ISBN:");
				System.out.println ("    java PruefzifferValidate -i -c 3823458529"); 
				System.out.println ("  transform an ISBN to EAN13:");
				System.out.println ("    java PruefzifferValidate -i -e 38234585291"); 
			
				disclaimer (11 + offset);
				break;
			case 2:  // Help for EAN13 calculation and validation
				System.out.println ("\nEAN13 validation/calculation");
				System.out.println ("\n  use -v for validation, -c for calculation or -e for extraction of ISBN");
				System.out.println ("\n  e.g. validate an EAN:");
				System.out.println ("    java PruefzifferValidate -e -v 4395431705508");
				System.out.println ("  calculate the checkdigit for an EAN:");
				System.out.println ("    java PruefzifferValidate -e -c 439543170550"); 
				System.out.println ("  transform an EAN13 to ISBN (if it's a book-EAN:");
				System.out.println ("    java PruefzifferValidate -e -e 9783823458524"); 
			
				disclaimer (11 + offset);
				break;
			case 3:  // Help for creditcardnumber validation
				System.out.println ("\nCreditcard validation");
				System.out.println ("\n  use: java PruefzifferValidate -c creditcardinstitute creditcardnumber");
				System.out.println ("\n  e.g. validate a creditcardnumber:");
				System.out.println ("    java PruefzifferValidate -c Master 12345678912345");
				System.out.println ("\n  Supported creditcardinstitutes are:");
				System.out.println ("    - VISA");
				System.out.println ("    - Eurocard");
				System.out.println ("    - Mastercard");
				System.out.println ("    - American Express (Amex)");
				System.out.println ("    - Diners Club");
				System.out.println ("    - Discover");
				disclaimer (15 + offset);
				break;
			case 4:  // Help for bankaccountnumber validation
				System.out.println ("\nBankaccountnumber validation");
				System.out.println ("\n  use: java PruefzifferValidate -b [-options] banknumber accountnumber");
				System.out.println ("    e.g.: java PruefzifferValidate -b 20050050 1212121");
				System.out.println ("\n  there a few options:");
				System.out.println ("    -a / -c            for caching 'all' or 'methodnumbers' of bankdata");
				System.out.println ("    -b                 for batch-validation");
				System.out.println ("    -l / -n / -s / -r  for low, normal, strict or restrictive strictness");
				System.out.println ("    e.g.: java PruefzifferValidate -b -b -c input.txt output.txt");
				System.out.println ("       for batch-validation of input.txt with caching of methodnumbers");
				System.out.println ("       and output to output.txt");
				System.out.println ("    for batch-validation the input-file must be formatted like this:");
				System.out.println ("       banknumber[whitespace]accountnumber[newline]");
				
				disclaimer (15 + offset);
				break;
		}
		
	}
	
	private void disclaimer (int lines) {
		
		for (int i = 0; i < (20 - lines); i++)
			System.out.println ();

		System.out.println (AccountValidator.MODULE_NAME + " " + AccountValidator.VERSION + "\n  " + AccountValidator.COPYRIGHT + "\n");
		
	}
	
		
	private String[] splitString (String toSplit, String delimiter) {
		
		String[] returnValue = {"",""};
		
		if (toSplit.length () <= 0)
			return returnValue;
			
		int i = toSplit.indexOf (delimiter);
		
		if (i == -1)
			returnValue[0] = toSplit;
		else {
			returnValue[0] = toSplit.substring (0,i);
			returnValue[1] = toSplit.substring (i+1);
		}	
		return returnValue;
	}

	private void listDataFileInformation () {
  		// New instance of AccountValidator-Class
		DatafileValidator ValidateDatafile = new DatafileValidator();
		java.text.DateFormat dateFormater = java.text.DateFormat.getDateInstance (java.text.DateFormat.LONG, java.util.Locale.US);
		
		boolean validData = ValidateDatafile.checkDataFileIntegrity ();
		
		System.out.println ();
		System.out.println ("The used banknumberfile is " + (validData?"VALID":"NOT VALID"));
		System.out.println ((validData?"":ValidateDatafile.getLastErrorText ()));
		System.out.println ("  --- actual banknumberfile --------");
		System.out.println ("    no.            " + ValidateDatafile.getDataUpdatenumber ());
		System.out.println ("    valid since    " + dateFormater.format (ValidateDatafile.getValidDate().getTime()));
		System.out.println ("  --- next banknumberfile ----------");
		System.out.println ("    no.            " + ValidateDatafile.getDataNextUpdatenumber ());
		System.out.println ("    will appear at " + dateFormater.format (ValidateDatafile.getExpireDate().getTime()));
		System.out.println ("  --- CRC32 validation -------------");
		System.out.println ("    deposited in file    " + ValidateDatafile.getFileCrc32 ());
		System.out.println ("    calculated from file " + ValidateDatafile.getCalculatedCrc32 ());
		System.out.println ("  ----------------------------------");
		
		disclaimer (13);
	}
	
	private void fileCheck (AccountValidator validator, String filename, String outputFilename) {

		if (filename.length () == 0)
			return;
			
		int counter = 0;

		try {
			BufferedReader in = new BufferedReader (new InputStreamReader( new FileInputStream (filename)));
    		BufferedWriter out = new BufferedWriter ( new OutputStreamWriter( new FileOutputStream (outputFilename)));
			try {
				String line;
				while ((line = in.readLine ()) != null) {
					
					if (++counter % 10000 == 0) {
						System.out.println ("Bearbeite Zeile " + counter);
						out.flush ();
					}

					if (line.length () > 0) {
						String[] data = splitString (line," ");
					
						boolean result = validator.validate (data[0], data[1]);
//						System.out.println (String.valueOf (result) + ((result)?" ":"") + " " + data[0] + "-" + data[1] + " " + validator.getUsedTestingMethod () + " " + validator.getLastErrorText ());
						out.write (String.valueOf (result) + ((result)?" ":"") + " " + line + " " + validator.getUsedTestingMethod ()); // + " " + validator.getBankName ());
						out.newLine ();
					}
					
				}
			} catch (IOException e) {
				System.out.println ("Datei blz.txt nicht gefunden");	
			}
			
			try {
				in.close ();
				out.close ();
			} catch (IOException e) {
				System.out.println ("Ein- / Ausgabedatei konnten nicht geschlossen werden");
			}
		} catch (FileNotFoundException e) {
    	  System.out.println ("Eingabedatei " + filename + " wurde nicht gefunden");
		}
	}	
}
